import click
import functools
from . import export, fetch


def get_username_password(config_file):
    import configparser
    config = configparser.ConfigParser()
    config.read(config_file)

    username = config['account']['username']
    password = config['account']['password']

    account = {
            'username': username,
            'password': password}
    return account


def common_options(func):
    @click.option('--debug/--no-debug', default=False)
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    return wrapper


@click.group()
def cli():
    pass


@cli.command()
@click.option('--config-file', default="config.ini", required=False)
@click.option('--trades', default="stock",
    type=click.Choice(['stock', 'option']), required=False)
@click.option('--duration', default="1m")
@click.option('--export-file', default="default.csv", required=False)
@common_options
def export_history(debug, duration, config_file, trades, export_file):
    if debug and config_file == "config.ini":
        config_file = "config.debug.ini"
    account = get_username_password(config_file)

    if trades == 'stock':
        stock_orders = fetch.stock_orders(account, {})
        click.echo("-- sm: Fetched {} stock orders".format(len(stock_orders)))

        click.echo("-- sm: Exporting to stock_orders.cvs")
        export.stock_orders(stock_orders, {})

    elif trades == 'option':
        option_orders = fetch.option_orders(account, {})
        click.echo("-- sm: Fetched {} option orders".format(len(option_orders)))

        click.echo("-- sm: Exporting to option_orders.csv")
        export.option_orders(option_orders, {})

    click.echo("-- sm: Finished")


if __name__ == '__main__':
    cli()



import csv
from . import fetch

def stock_trades(account, options):
    stock_orders = fetch.stock_trades(account, options)
    write_rows_of_dictionaries()

    print('exporting RH stock history')

def option_orders(option_orders, options = {}):
    drows = option_orders
    cols = [*drows[0].keys()]
    headers = order_option_cols(cols)
    filename = "option_orders.csv"

    try:
        with open(filename, 'w') as csvfile:
            writer = csv.DictWriter(csvfile, fieldnames=headers)
            writer.writeheader()
            [writer.writerow(drow) for drow in drows]
    except IOError:
        print("I/O error")


def stock_orders(stock_orders, options = {}):
    drows = stock_orders
    cols = [*drows[0].keys()]
    headers = ordered_stock_cols(cols)
    filename = "stock_orders.csv"

    try:
        with open(filename, 'w') as csvfile:
            writer = csv.DictWriter(csvfile, fieldnames=headers)
            writer.writeheader()
            [writer.writerow(drow) for drow in drows]
    except IOError:
        print("I/O Error")


def order_option_cols(cols):
    expected_headers_ordered = [
        "id", "chain_id", "ref_id",  "updated_at", "created_at", "time_in_force",
        "chain_symbol", "state", "type", "direction", "price", "premium", "processed_premium",
        "quantity", "pending_quantity", "processed_quantity",
        "opening_strategy", "closing_strategy", "legs",
        "trigger", "response_category",
        "cancel_url", "canceled_quantity",
    ]

    if set(cols) == set(expected_headers_ordered):
        return expected_headers_ordered
    else:
        return cols


def ordered_stock_cols(cols):
    expected_headers_ordered = [
        'id', 'ref_id', 'instrument', 'updated_at', 'created_at', 'last_transaction_at',
        'time_in_force', 'trigger', 'cancel', 'response_category','state', 'reject_reason',
        'type', 'side', 'price', 'stop_price', 'average_price', 'quantity', 'cumulative_quantity', 'fees',
        'executions', 'extended_hours', 'account', 'url', 'position',
        'override_day_trade_checks', 'override_dtbp_checks',
    ]

    if set(cols) == set(expected_headers_ordered):
        return expected_headers_ordered
    else:
        return cols



from distutils.core import setup

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except(IOError, ImportError):
    long_description = open('README.md').read()


setup(name='simple_metrics',
    version='0.1.3',
    description='Simple portfolio metrics for Robinhood',
    long_description=long_description,
    author='Weston Platter',
    author_email='westonplatter@gmail.com',
    url='https://github.com/westonplatter/simple_metrics/',
    packages=['simple_metrics'],
    install_requires=[
        'click',
        'Robinhood==1.0.1',
    ],
    dependency_links=[
        # # this git commit hash is master as of July 14th. It's 64 commits ahead of release 1.0.1
        # 'https://github.com/Jamonek/Robinhood/archive/d21b1907dfa0ec9ba04177e597350b7d2a1ef31e.zip#egg=Robinhood-1.0.1',

        'https://github.com/westonplatter/Robinhood/archive/6294a5c3d53dd7553c05c272ebeb069df1c86aa2.zip#egg=Robinhood-1.0.1',
    ],
    entry_points='''
        [console_scripts]
        sm=simple_metrics.cli:cli
    ''',
)



from Robinhood import Robinhood


def fetch_json_by_url(client, url):
    return client.session.get(url).json()


def stock_orders(account, options={}):
    client = Robinhood()
    client.login(username=account['username'], password=account['password'])

    stock_orders = []
    more_orders = True

    res = client.order_history()
    stock_orders.extend(res['results'])
    cursor = res['next']
    while cursor:
        res = fetch_json_by_url(client, cursor)
        stock_orders.extend(res['results'])
        cursor = res['next']

    return stock_orders


def option_orders(account, options={}):
    client = Robinhood()
    client.login(username=account['username'], password=account['password'])

    options_orders = []
    more_orders = True

    res = client.options_order_history()
    options_orders.extend(res['results'])
    cursor = res['next']

    while cursor:
        res = fetch_json_by_url(client, cursor)
        options_orders.extend(res['results'])
        cursor = res['next']

    return options_orders
